/* @(#)cgna.c	14.1.1.2 (ESO-DMD) 05/10/00 23:21:18 */
/*===========================================================================
  Copyright (C) 1995 European Southern Observatory (ESO)
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 675 Massachusetts Ave, Cambridge, 
  MA 02139, USA.
 
  Correspondence concerning ESO-MIDAS should be addressed as follows:
	Internet e-mail: midas@eso.org
	Postal address: European Southern Observatory
			Data Management Division 
			Karl-Schwarzschild-Strasse 2
			D 85748 Garching bei Muenchen 
			GERMANY
===========================================================================*/

/*+++++++++++++++++++ CGN interfaces part A  +++++++++++++++++++++++++++++++
.LANGUAGE   C
.IDENTIFICATION Module CGNA
.COMMENTS
  holds  CLEANF, INDEXC, INDEXS
.AUTHOR  K. Banse	ESO - Garching
.KEYWORDS
  name translation, parsing, filling
.ENVIRONMENT VMS and UNIX
.VERSION
 [1.40]	870715: C-version
 [1.70] 881011: take partition stuff out completely + use CGN_LOGNAM
 [1.80] 890414: try to fix the ".." business in CGN_EXTRSS
 [1.90] 891024: modify CLEANF for file[...] or file@no
 [2.10] 900725: use register variables + clean up a bit
 [2.20] 910226: omit error display in CGN_DISPFIL
 [2.30] 930907: long int -> int !!!
 [2.40] 940125: throw out check for special `bad' characters !!!

000316
-----------------------------------------------------------------------------*/
 
 
#include <fsydef.h>
#include <midas_def.h>
#include <fileexts.h>
#include <osfile.h>

 

/*

*/

CGN_CLEANF(infile,deftypno,outfile,lout,namtype,ext_flg)
 
/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE
  translate file name and append default type, if required.
  also indicate, if extracted file is wanted
  cleaned filename is terminated by \0
.ALGORITHM
  straight forward via CGN_INDEXC
  we have to use FSYDEF.H, which defines file-system dependant values  !!
.RETURNS
  return status  = 0, if all o.k.
                 = -1, if bad syntax
------------------------------------------------------------------------*/
 
char   *infile;		/* IN : (complete) file name (terminated by \0) */
int    deftypno;	/* IN : default_type_no. 0 = file type will decide 
                                1 = image (.bdf), 
  		                3 = table (.tbl), 4 = fit file (.fit) 
			        5 = logfile (.log), 
 			        6 = procedure (.prg),
			        7 = catalog (.cat)        */
int    lout;		/* IN : max length of outfile */
char   *outfile;	/* OUT: "cleaned" file name */
int    *namtype;	/* OUT: type of frame:                        
                                1 = image (.bdf), 2 = any (.any)       
			        3 = table (.tbl), 4 = fit file (.fit) 
			        5 = logfile (.log), 6 = procedure (.prg) 
			        7 = catalog (.cat), 8 = test (.tst)        
			        9 = executable (.exe)    
			        0 = if something wrong...    */
int    *ext_flg;	/* OUT: = 0, if none; 
			        > 0 if index of '[' or '@' 
			        < 0 if index of [extens]   */


{
int   k, dd, tt, nn, inlen;
register int nr;

char	work[200];
char	*defpntr, *cpntr;




*ext_flg = 0;
*namtype = 0;

if ((deftypno < 1) || (deftypno > 7))
   nn = 0;
else 
   nn = deftypno - 1;
defpntr = FSY_DEFPNTR[nn];		/* point to correct deftype  */

inlen = (int)strlen(infile);


/* check if name is enclosed in special char. XFNAMC (currently = ")
   then we just strip them off  ...  */

if (*infile == XFNAMC)
   {
   k = inlen - 1;
   if (infile[k] == XFNAMC)                /* it's "name" */
      {
      if (lout < k) 
         k = lout - 1;
      else
         k --;                             /* new lenght of string */
      (void) strncpy(outfile,&infile[1],k);
      outfile[k] = '\0';
      *namtype = 99;                    /* we don't check ... */
      return (0);
      }
   }


/* first look for file[... : ...] or file[extens] */

cpntr = infile;
if (infile[inlen-1] == ']')
   {
   k = CGN_JNDEXC(infile,'[');
   if (k < 1) return (-2);

   *ext_flg = k;
   (void) strncpy(work,infile,k);
   work[k] = '\0';
   cpntr = work;

   nn = CGN_INDEXC(&infile[k+1],':');
   if (nn < 1) *ext_flg = -k;		/* we have a FITS extension */
   }
         
/* now check for file@plane/line/pixel */

else
   {		
   k = CGN_INDEXC(infile,'@');
   if (k > 0)
      {
      *ext_flg = k;
      (void) strncpy(work,infile,k);
      work[k] = '\0';
      cpntr = work;
      }
   } 

CGN_LOGNAM(cpntr,outfile,lout);		/* returns outfile with \0  */

tt = CGN_JNDEXC(outfile,FSY_TYPMARK);	/* now, look for type */
dd = CGN_JNDEXC(outfile,FSY_DIREND);	/* and directory endmarker */

if (dd < 0) dd = 0;			/* no direspecs */
if (tt < dd)
   {
   (void) strcat(outfile,defpntr);		/* append def type to file */
   tt = -1;
   }

/* printf("CGN_CLEANF: infile = %s - outfile = %s\n",infile,outfile); */
 
 
if (deftypno == 0)
   {
   if (tt > -1)
      {
      cpntr = outfile + tt;
      for (nr=0; nr<9; nr++)
         {
         if ( strcmp(cpntr,FSY_DEFPNTR[nr]) == 0)
            {
            *namtype = nr + 1;
            return(0);
            }
         }
      }
   *namtype = 1;				/* default to images */
   }

else
   *namtype = deftypno;


return (0);
}	

/*

*/

#ifdef __STDC__
int CGN_INDEXC(char *s, char t)
#else
int CGN_INDEXC(s,t)
/*++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE
 returns position of single character in input string
.RETURN
 returns position of single character in input string, -1 if not there
--------------------------------------------------*/

char *s;    /* input string  */
char t;     /* test character */
#endif

{
register int i;

char   *cp;


cp = s;
for (i=0; *cp != '\0'; i++)
   {
   if (t == *cp++) return (i);
   }

return (-1);			/* character not found */
}

/*

*/

CGN_INDEXS(s,t)
 
/*++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE
 find position of substring in input string.

.RETURN
 returns index of substring in input string (= 0,1,2,...),
         -1 if not there
--------------------------------------------------*/
 
char *s;    /* input string  */
char *t;    /* substring */

{
register int nr;

register char  *cp, *cq, *qq, *t2;


cp = s;
t2 = t + 1;
 
for (nr=0; *cp != '\0'; nr++)
   {
   if (*cp++ == *t)			/* first char. has to match */
      {
      qq = cp;				/* points to 2. char in input string */
      for (cq=t2; *cq != '\0'; )
         {
         if (*cq++ != *qq++) goto no_match;
         }
      return (nr);
      }
no_match:
   ;
   }
 
return (-1);			/* substring not found */
}
 
/*

*/

CGN_FILINFO(file,size,time,prot)

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.PURPOSE
  get file info
.ALGORITHM
  use osfinfo
.RETURNS
  return status  = 0, if all o.k.
                 = -1, if file could not be accessed
------------------------------------------------------------------------*/
char *file;
long int *size, *time;
int  *prot;

{
int  n;

struct filestatus fstat;



n = osfinfo(file,&fstat);
if (n != 0) return (-1);

*size = fstat.filesize;
*time = (long int) fstat.date;
*prot = fstat.protection;

return 0;
}

